action=$1
task=$2

if [[ "$action" == "powersave" || "$action" == "balance" ]] && [[ $(cat /sys/class/power_supply/usb/online) == 1 ]];then
  action='performance'
fi

cfg_dir=$(cd $(dirname $0); pwd)

if [[ ! -f "$cfg_dir/powercfg-utils.sh" ]]; then
  echo "The dependent '$cfg_dir/powercfg-utils.sh' was not found !" > /cache/powercfg.sh.log
  exit 1
fi

source "$cfg_dir/powercfg-utils.sh"

init () {
  if [[ -f "$cfg_dir/powercfg-base.sh" ]]; then
    source "$cfg_dir/powercfg-base.sh"
  elif [[ -f '/data/powercfg-base.sh' ]]; then
    source /data/powercfg-base.sh
  fi
}

if [[ "$action" == "init" ]]; then
  init
  exit 0
fi

reset_basic_governor

# dump_cluster_0_dvfs_table
# 2000000 1800000 1725000 1625000 1525000 1450000 1350000 1250000 1075000 1000000 925000 850000 750000 675000 600000 500000
# dump_cluster_1_dvfs_table
# 2600000 2507000 2354000 2200000 1985000 1855000 1740000 1624000 1537000 1451000 1335000 1162000 1046000 902000 700000 437000
# dump_cluster_2_dvfs_table
# 3000000 2892000 2713000 2600000 2463000 2284000 2141000 1998000 1820000 1632000 1482000 1370000 1258000 1108000 921000 659000

if [[ "$action" = "powersave" ]]; then
  #powersave

  hmp
  min_freq 500000 437000 659000
  max_freq 1725000 1622000 1632000
  cpu_cci_mode 0
  cpu_power_mode 1
  dram_freq 0
  sched_limit 0 0 0 0 0 0
  stune_top_app 0 0
  stune_util background 0 0
  stune_util foreground 1024 0
  stune_util top-app 1024 0

  # ged gpu_dvfs_enable 1
  gpu_dvfs_margin 10
  # ged gx_game_mode 0

  cpuset 0-2 0-3 0-7 0-7 0-3

  tcp_low_latency 0
  realme_gt_on=0

elif [[ "$action" = "balance" ]]; then
  #balance

  hybrid
  min_freq 500000 437000 659000
  max_freq 1800000 1985000 2284000
  cpu_cci_mode 0
  cpu_power_mode 0
  dram_freq 0
  sched_limit 0 0 0 0 0 0
  stune_top_app 0 0
  stune_util background 1024 0
  stune_util foreground 1024 0
  stune_util top-app 1024 0

  # ged gpu_dvfs_enable 1
  # ged gx_game_mode 0
  gpu_dvfs_margin 10

  cpuset 0-2 0-3 0-7 0-7 0-3

  tcp_low_latency 0
  realme_gt_on=0

elif [[ "$action" = "performance" ]]; then
  #performance

  hybrid
  min_freq 500000 437000 659000
  max_freq 2000000 2354000 2713000
  cpu_cci_mode 1
  cpu_power_mode 3
  dram_freq 0
  sched_limit 5000 0 500 0 500 0
  stune_top_app 0 10
  stune_util background 1024 0
  stune_util foreground 1024 50
  stune_util top-app 1024 200

  # ged gpu_dvfs_enable 1
  # ged gx_game_mode 1
  gpu_dvfs_margin 18

  cpuset 0-1 0-3 0-7 0-7 0-3
  sched_isolation_disable

  tcp_low_latency 1
  realme_gt_on=1

elif [[ "$action" = "fast" ]]; then
  #fast

  hybrid
  min_freq 1725000 1624000 1820000
  max_freq 2000000 2600000 3000000
  cpu_cci_mode 1
  cpu_power_mode 3
  dram_freq max
  sched_limit 10000 0 2000 0 2000 0
  stune_top_app 1 20
  stune_util background 1024 0
  stune_util foreground 1024 200
  stune_util top-app 1024 1014

  # ged gpu_dvfs 1
  # ged gx_game_mode 0
  gpu_dvfs_margin 25

  cpuset 0 0-3 0-7 0-7 0-3
  sched_isolation_disable

  tcp_low_latency 1
  realme_gt_on=1

elif [[ "$action" = "pedestal" ]]; then
  hybrid
  min_freq 2000000 2600000 2600000
  max_freq 2000000 2600000 3000000
  cpu_cci_mode 1
  cpu_power_mode 3
  dram_freq max
  sched_limit 10000 0 20000 0 20000 0
  stune_top_app 1 100
  stune_util background 1024 0
  stune_util foreground 1024 200
  stune_util top-app 1024 1014

  # ged gpu_dvfs_enable 1
  # ged gx_game_mode 0
  gpu_dvfs_margin 130

  cpuset 0 0-3 0-7 0-7 0-3
  sched_isolation_disable

  tcp_low_latency 1
  # sspm_thermal_throttle 1
  realme_gt_on=1

fi

adjustment_by_top_app
realme_gt $realme_gt_on
